@@ -1,18 +1,34 @@
+import argparse
+import dask.array as da
 import logging
 import sys
 import torch
 import torch.nn as nn
 import wandb
 from codetiming import Timer
-from model.neural_admixture import NeuralAdmixture
-from model.switchers import Switchers
+from ..model.neural_admixture import NeuralAdmixture
+from ..model.switchers import Switchers
 from pathlib import Path
-from src import utils
+from . import utils
+from typing import List, Tuple, Union
 
 logging.basicConfig(stream=sys.stdout, level=logging.INFO)
 log = logging.getLogger(__name__)
 
-def fit_model(trX, args, valX=None, trY=None, valY=None):
+def fit_model(trX: da.core.Array, args: argparse.Namespace, valX: Union[None, da.core.Array]=None,
+              trY: Union[None, List[str]]=None, valY: Union[None, List[str]]=None) -> Tuple[NeuralAdmixture, torch.device]:
+    """Wrapper function to start training
+
+    Args:
+        trX (da.core.Array): Dask array containing training data.
+        args (argparse.Namespace): parsed argument from CLI.
+        valX (Union[None, da.core.Array], optional): Dask array containing validation data. Defaults to None.
+        trY (Union[None, List[str]], optional): list containing training labels. Defaults to None.
+        valY (_type_, optional): list containing validation labels. Defaults to Union[None, List[str]]=None.
+
+    Returns:
+        Tuple[NeuralAdmixture, torch.device]: instantiated model object along with device.
+    """
     switchers = Switchers.get_switchers()
     num_max_epochs = args.max_epochs
     batch_size = args.batch_size
@@ -53,12 +69,12 @@ def fit_model(trX, args, valX=None, trY=None, valY=None):
     # Initialization
     log.info('Initializing...')
     if init_file is None and decoder_init != "pretrained":
-        log.warning(f'Initialization filename not provided. Going to store it to {save_dir}/{run_name}.pkl')
+        log.warning(f'Initialization filename not provided. Going to store it to {Path(save_dir)/run_name}.pkl')
         init_file = f'{run_name}.pkl'
-    init_path = f'{save_dir}/{init_file}' if decoder_init != "pretrained" else init_file
-    P_init = switchers['initializations'][decoder_init](trX, trY, Ks, seed, init_path, run_name, n_components)
+    init_path = f'{Path(save_dir)/init_file}' if decoder_init != "pretrained" else init_file
+    P_init = switchers['initializations'][decoder_init](trX, trY, Ks, seed, init_path, run_name, n_components, batch_size)
     activation = switchers['activations'][activation_str](0)
-    log.info('Variants: {}'.format(trX.shape[1]))
+    log.info(f'Variants: {trX.shape[1]}')
     model = NeuralAdmixture(Ks, trX.shape[1], P_init=P_init,
                                 lambda_l2=l2_penalty,
                                 encoder_activation=activation,
@@ -69,7 +85,7 @@ def fit_model(trX, args, valX=None, trY=None, valY=None):
     model.to(device)
     if log_to_wandb:
         wandb.watch(model, log='all', log_freq=1000)
-    
+
     # Optimizer
     optimizer = switchers['optimizers'][optimizer_str](filter(lambda p: p.requires_grad, model.parameters()), learning_rate)
     log.info('Optimizer successfully loaded.')
@@ -93,12 +109,14 @@ def fit_model(trX, args, valX=None, trY=None, valY=None):
     log.info('Optimization process finished.')
     return model, device
 
-def main(argv):
+def main(argv: List[str]):
+    """Training entry point
+    """
     args = utils.parse_train_args(argv)
     tr_file, val_file = args.data_path, args.validation_data_path
     tr_pops_f, val_pops_f = args.populations_path, args.validation_populations_path
-
     trX, trY, valX, valY = utils.read_data(tr_file, val_file, tr_pops_f, val_pops_f)
+    print(type(trX))
     model, device = fit_model(trX, args, valX, trY, valY)
     log.info('Computing divergences...')
     model.display_divergences()
